#!/usr/bin/env python
# -*- coding: euc-jp -*-

import sys, re, os
from getopt import getopt

from simpleparse.parser import Parser

import common, tmpl_html

# ǥե
config = {
	'-o': 'result.html',
	'--hackdir': '../nethack-3.4.3',
	'--monst': 'src/monst.c',
	'--jtrnsmon': 'dat/jtrnsmon.dat',
	'--def': 'monst.def',
}

def alert(msg):
	print >> sys.stderr, unicode(msg, 'euc-jp')

def translate_table(dat_file):
	dic = {}
	
	for line in open(dat_file):
		m = re.match(r'(?P<en>[^#][^:]+):\s+(?P<jp>[^:]+):?', line.rstrip('\r\n'))
		if m:
			dic[m.group('en')] = m.group('jp')
	
	return dic


def monst_analysis(monstdata):
	# Ͽ٤
	valid_tags = ['name', 'symbol', 'geno', 'lvl', 'move_rate', 'AC', 'MR', 'align',
	'weight', 'nutrition', 'ext_length', 'sound', 'physical', 'a', 'resist',
	'resist_confer', 'flag_M1', 'flag_M2', 'flag_M3', 'color',
	
	'roguemonst', 'roguemonst_attr', 'atk1', 'atk2', 'atk3', 'atk4'] # NetHack tNG
	
	# 󥹥ǡΥꥹ
	monsters = []
	
	def tree_traverse(L):
		for (tag, begin, end, child) in L:
			if tag in valid_tags:
				# Ͽ
				current[tag] = monstdata[begin:end]
			if child:
				# ȤդǤʤкƵŪŬ
				tree_traverse(child)
	
	if not os.path.exists(config['--def']):
		alert('顼: [%(--def)s]¸ߤޤ' % config)
		sys.exit()
		
	decl = ''.join(open(config['--def']))
	parser = Parser(decl, 'all')
	attack_parser = Parser(decl, 'a_detail')
	
	succeed, tree, offset = parser.parse(monstdata)
	
	if succeed and (offset == len(monstdata)):
		# ޥå
		for (tag, begin, end, child) in tree:
			if tag == 'mondata':
				monsters.append({})
				current = monsters[-1]
				tree_traverse(child)
				
		for monster in monsters:
			if 'a' in monster:
				monster['a'] = attack_analysis(monster['a'], attack_parser)
				
			# NetHack tNGѽ
			if 'roguemonst' in monster:
				monster.update(roguemonst_attr = '', flag_M1 = 'M1_ANIMAL', flag_M2 = 'M2_HOSTILE', flag_M3 = '0')
			if 'roguemonst_attr' in monster:
				monster.update(
					move_rate = '10',
					AC = str(10 - int(monster['AC'])),
					MR = '0', align = '0',
					geno = '(G_GENO|G_ROGUE|G_NOCORPSE|2)',
					a = attack_analysis('A(ATTK(%(atk1)s,%(atk2)s,%(atk3)s,%(atk4)s))' % monster, attack_parser),
					weight = '0', nutrition = '0', ext_length = '0',
					sound = 'MS_SILENT',
					physical = 'MZ_MEDIUM',
					resist = '0', resist_confer = '0',
					color = 'CLR_GRAY')
						
		return monsters
	else:
		alert('Ϥ˼Ԥޤ')


def attack_analysis(text, parser):
	if text == 'SEDUCTION_ATTACKS':
		# Х/󥭥ХͶǹ(ȤꤢǤ)
		return 'դ(Ͷ), ä(ʪ)d3, ä(ʪ)d3'
		
	valid_tags = ['type', 'dmg_type', 'dices', 'sides']
	attacks = []
	
	def tree_traverse(L):
		for (tag, begin, end, child) in L:
			if tag in valid_tags:
				# Ͽ
				current[tag] = text[begin:end]
			if child:
				# ȤդǤʤкƵŪŬ
				tree_traverse(child)
				
	succeed, tree, offset = parser.parse(text)
	if succeed and (offset == len(text)):
		# ޥå
		for (tag, begin, end, child) in tree:
			if tag == 'attack':
				attacks.append({})
				current = attacks[-1]
				tree_traverse(child)
				
		for (i, attack) in enumerate(attacks):
			if attack:
				# 
				if attack['type'] in common.dic:
					attack['type'] = common.dic[attack['type']]
				else:
					alert('ٹ: [%s]Ǥ' % attack['type'])
					
				# ᡼
				if attack['dmg_type'] in common.dic:
					attack['dmg_type'] = common.dic[attack['dmg_type']]
				else:
					alert('ٹ: [%s]Ǥ' % attack['dmg_type'])
					
				# 0d0 -> ''
				# 1d6 -> 'd6'
				# 2d6 -> '2d6'
				if attack ['dices'] == attack['sides'] == '0':
					d = ''
				elif attack['dices'] in ['0', '1']:
					d = 'd%(sides)s' % attack
				else:
					d = '%(dices)sd%(sides)s' % attack
					
				if attack['dmg_type'] != '0':
					attacks[i] = ('%(type)s(%(dmg_type)s)' % attack) + d
				else:
					attacks[i] = ('%(type)s' % attack) + d
			else: #NO_ATTK
				attacks[i] = ''
				
		return ', '.join(s for s in attacks if s)
	else:
		return text


def flag_analysis(text):
	if text in ['', '0']:
		return '  '
	else:
		flags = []
		
		for flag in (s for s in re.split('[|\(\) \t\n]', text) if s):
			if flag == '0':
				pass
			elif flag in common.dic:
				flags.append(common.dic[flag])
			else:
				alert('ٹ: [%s]Ǥ' % flag)
		
		return ', '.join(flags)


def format(dic, en2jp):
	# ̾
	dic['name'] = dic['name'].replace('"', '')
	if dic['name'] in en2jp:
		dic['jpname'] = en2jp[dic['name']]
	else:
		dic['jpname'] = dic['name']
	
	# ܥ
	if dic['symbol'] in common.dic:
		dic['symbol'] = common.dic[dic['symbol']]
	else:
		alert('ٹ: [%(name)s]Υܥ[%(symbol)s]Ǥ' % dic)
		
	# 
	if dic['color'] in common.dic:
		dic['color'] = common.dic[dic['color']]
	else:
		alert('ٹ: [%(name)s]ο[%(color)s]Ǥ' % dic)
		
	# "󥰥ο"
	if dic['name'] == 'long worm tail':
		dic['lvl'] = dic['move_rate'] = dic['AC'] = dic['MR'] = dic['align'] = \
		dic['weight'] = dic['nutrition'] = dic['physical'] = dic['sound'] =    \
		dic['resist'] = dic['resist_confer'] = dic['a'] = dic['geno'] =        \
		dic['flag_M1'] = dic['flag_M2'] = dic['flag_M3'] = ''
		return
	
	# armor class
	dic['AC'] = re.sub(r'MARM\((\d+),\d+\)', r'\1', dic['AC'])
	
	# °
	
	dict
	
	
	if dic['align'] == 'A_NONE':
		# "ˡȤ"
		# "̵°"ȤΤ⤷ʤİǴƤ""ɽΤ:
		dic['align'] = ''
	elif int(dic['align']) == 0:
		dic['align'] = 'Ω'
	elif int(dic['align']) > 0:
		dic['align'] = '(%(align)s)' % dic
	elif int(dic['align']) < 0:
		dic['align'] = '(%(align)s)' % dic
	
	# 
	if dic['weight'].startswith('WT_'):
		try:
			dic['weight'] = { 'WT_HUMAN':  'ʹ(1450)',
							  'WT_ELF':    '(800)',
							  'WT_DRAGON': 'ɥ饴(4500)'}[dic['weight']]
		except KeyError:
			alert('ٹ: [%(name)s]ν[%(weight)s]Ǥ' % dic)
	
	# 
	if dic['physical'] in common.dic:
		dic['physical'] = common.dic[dic['physical']]
	elif dic['physical'] != '0':
		alert('ٹ: [%(name)s]Υ[%(physical)s]Ǥ' % dic)
	
	# ư
	if dic['sound'] in common.dic:
		dic['sound'] = common.dic[dic['sound']]
	else:
		alert('ٹ: [%(name)s]ιư[%(sound)s]Ǥ' % dic)
	
	dic['geno'] = flag_analysis(dic['geno'])
	dic['resist'] = flag_analysis(dic['resist'])

	# "ְִư"
	if 'M1_TPORT' in dic['flag_M1']:
		dic['resist_confer'] += '|M1_TPORT'
	# "ְִư"
	if 'M1_TPORT_CNTRL' in dic['flag_M1']:
		dic['resist_confer'] += '|M1_TPORT_CNTRL'
	# "ESP"
	if dic['name'] in ['floating eye', 'mind flayer', 'master mind flayer']:
		dic['resist_confer'] += '|telepathy'
	
	# Ǥpermonst.mconveysMR_STONEΩäƤƤä˲ʤΤ̵
	dic['resist_confer'] = flag_analysis(dic['resist_confer'].replace('MR_STONE', ''))
	
	dic['flag_M1'] = flag_analysis(dic['flag_M1'])
	dic['flag_M2'] = flag_analysis(dic['flag_M2'])
	dic['flag_M3'] = flag_analysis(dic['flag_M3'])


def main(args):
	optlist, args = getopt(args, 'o:v', ['hackdir=', 'monst=', 'jtrnsmon=', 'def='])
	config.update(optlist)
	
	if not os.path.exists('%(--hackdir)s/%(--monst)s' % config):
		alert('顼: [%(--hackdir)s/%(--monst)s]¸ߤޤ' % config)
		sys.exit()
	
	monstdata = open('%(--hackdir)s/%(--monst)s' % config).readlines();
	# Ѵơ֥
	en2jp = {}
	if os.path.exists('%(--hackdir)s/%(--jtrnsmon)s' % config):
		en2jp = translate_table('%(--hackdir)s/%(--jtrnsmon)s' % config)
	
	ignore = 0 # ignore > 0 ʤɤФ
	commentout = re.compile('(#if 0|#ifdef SEDUCE|#ifdef DWARF|#ifdef CHARON).*')
	ccomment = re.compile(r'/\*(?!\*/).*?\*/')
	endif = re.compile(r'#[ \t]*endif.*')

	for (i, line) in enumerate(monstdata):
		if(commentout.match(monstdata[i])):
			ignore += 1
			
		if ignore or line.startswith('#'):
			monstdata[i] = ''
			if endif.match(line):
				ignore = max(0, ignore - 1)
				if ('*/' in line) and (not '/*' in line): # for '#endif */', not for '#endif /* 0 */'
					monstdata[i] = line
		else:
			monstdata[i] = line.rstrip('\r\n')

	# C륳Ȥ
	monstdata = ccomment.sub('', ''.join(monstdata))
	
	data = re.search(r'mons\[\] = \{[^\}]*\}', monstdata)
	data2 = re.search(r'mons2\[\] = \{[^\}]*?\}', monstdata)

	monstdata = data.group() + data2.group()
	
	if not monstdata:
		alert('顼: Ϥ˼Ԥޤ')
		sys.exit()
	
	# ǡ񤭽Ф(ʼ)
	monsters = monst_analysis(monstdata)[:-1]
	
	if monsters:
		f = open(config['-o'], 'w')
		f.write(tmpl_html.template['header'])
		
		for data in monsters:
			format(data, en2jp)
			f.write(tmpl_html.template['body'] % data)
	
		f.write(tmpl_html.template['footer'])
		f.close()

if __name__ == '__main__':
	main(sys.argv[1:])